@@ -5,18 +5,23 @@
 
     This module provides various traceback related utility functions.
 
-    :copyright: (c) 2011 by the Werkzeug Team, see AUTHORS for more details.
+    :copyright: (c) 2013 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD.
 """
 import re
+
 import os
 import sys
+import json
 import inspect
 import traceback
 import codecs
 from tokenize import TokenError
+
 from werkzeug.utils import cached_property, escape
 from werkzeug.debug.console import Console
+from werkzeug._compat import range_type, PY2, text_type, string_types
+
 
 _coding_re = re.compile(r'coding[:=]\s*([-\w.]+)')
 _line_re = re.compile(r'^(.*?)$(?m)')
@@ -37,12 +42,17 @@ HEADER = u'''\
   <head>
     <title>%(title)s // Werkzeug Debugger</title>
     <link rel="stylesheet" href="?__debugger__=yes&amp;cmd=resource&amp;f=style.css" type="text/css">
+    <!-- We need to make sure this has a favicon so that the debugger does not by
+         accident trigger a request to /favicon.ico which might change the application
+         state. -->
+    <link rel="shortcut icon" href="?__debugger__=yes&amp;cmd=resource&amp;f=console.png">
     <script type="text/javascript" src="?__debugger__=yes&amp;cmd=resource&amp;f=jquery.js"></script>
     <script type="text/javascript" src="?__debugger__=yes&amp;cmd=resource&amp;f=debugger.js"></script>
     <script type="text/javascript">
       var TRACEBACK = %(traceback_id)d,
           CONSOLE_MODE = %(console)s,
-          EVALEX = %(evalex)s
+          EVALEX = %(evalex)s,
+          SECRET = "%(secret)s";
     </script>
   </head>
   <body>
@@ -66,11 +76,12 @@ PAGE_HTML = HEADER + u'''\
 <h2 class="traceback">Traceback <em>(most recent call last)</em></h2>
 %(summary)s
 <div class="plain">
-  <form action="%(lodgeit_url)s" method="post">
+  <form action="/?__debugger__=yes&amp;cmd=paste" method="post">
     <p>
       <input type="hidden" name="language" value="pytb">
       This is the Copy/Paste friendly version of the traceback.  <span
-      class="pastemessage">You can also paste this traceback into LodgeIt:
+      class="pastemessage">You can also paste this traceback into
+      a <a href="https://gist.github.com/">gist</a>:
       <input type="submit" value="create paste"></span>
     </p>
     <textarea cols="50" rows="10" name="code" readonly>%(plaintext)s</textarea>
@@ -127,11 +138,12 @@ SOURCE_LINE_HTML = u'''\
 '''
 
 
-def render_console_html():
+def render_console_html(secret):
     return CONSOLE_HTML % {
         'evalex':           'true',
         'console':          'true',
         'title':            'Console',
+        'secret':           secret,
         'traceback_id':     -1
     }
 
@@ -146,7 +158,7 @@ def get_current_traceback(ignore_system_exceptions=False,
     exc_type, exc_value, tb = sys.exc_info()
     if ignore_system_exceptions and exc_type in system_exceptions:
         raise
-    for x in xrange(skip):
+    for x in range_type(skip):
         if tb.tb_next is None:
             break
         tb = tb.tb_next
@@ -247,26 +259,45 @@ class Traceback(object):
     def exception(self):
         """String representation of the exception."""
         buf = traceback.format_exception_only(self.exc_type, self.exc_value)
-        return ''.join(buf).strip().decode('utf-8', 'replace')
+        rv = ''.join(buf).strip()
+        return rv.decode('utf-8', 'replace') if PY2 else rv
     exception = property(exception)
 
     def log(self, logfile=None):
         """Log the ASCII traceback into a file object."""
         if logfile is None:
             logfile = sys.stderr
-        tb = self.plaintext.encode('utf-8', 'replace').rstrip() + '\n'
+        tb = self.plaintext.rstrip() + u'\n'
+        if PY2:
+            tb.encode('utf-8', 'replace')
         logfile.write(tb)
 
-    def paste(self, lodgeit_url):
+    def paste(self):
         """Create a paste and return the paste id."""
-        from xmlrpclib import ServerProxy
-        srv = ServerProxy('%sxmlrpc/' % lodgeit_url)
-        return srv.pastes.newPaste('pytb', self.plaintext)
+        data = json.dumps({
+            'description': 'Werkzeug Internal Server Error',
+            'public': False,
+            'files': {
+                'traceback.txt': {
+                    'content': self.plaintext
+                }
+            }
+        }).encode('utf-8')
+        try:
+            from urllib2 import urlopen
+        except ImportError:
+            from urllib.request import urlopen
+        rv = urlopen('https://api.github.com/gists', data=data)
+        resp = json.loads(rv.read().decode('utf-8'))
+        rv.close()
+        return {
+            'url': resp['html_url'],
+            'id': resp['id']
+        }
 
     def render_summary(self, include_title=True):
         """Render the traceback for the interactive console."""
         title = ''
-        description = ''
         frames = []
         classes = ['traceback']
         if not self.frames:
@@ -296,20 +327,20 @@ class Traceback(object):
             'description':  description_wrapper % escape(self.exception)
         }
 
-    def render_full(self, evalex=False, lodgeit_url=None):
+    def render_full(self, evalex=False, secret=None):
         """Render the Full HTML page with the traceback info."""
         exc = escape(self.exception)
         return PAGE_HTML % {
             'evalex':           evalex and 'true' or 'false',
             'console':          'false',
-            'lodgeit_url':      escape(lodgeit_url),
             'title':            exc,
             'exception':        exc,
             'exception_type':   escape(self.exception_type),
             'summary':          self.render_summary(include_title=False),
             'plaintext':        self.plaintext,
             'plaintext_cs':     re.sub('-{2,}', '-', self.plaintext),
-            'traceback_id':     self.id
+            'traceback_id':     self.id,
+            'secret':           secret
         }
 
     def generate_plaintext_traceback(self):
@@ -356,7 +387,7 @@ class Frame(object):
         info = self.locals.get('__traceback_info__')
         if info is not None:
             try:
-                info = unicode(info)
+                info = text_type(info)
             except UnicodeError:
                 info = str(info).decode('utf-8', 'replace')
         self.info = info
@@ -405,13 +436,11 @@ class Frame(object):
 
     def eval(self, code, mode='single'):
         """Evaluate code in the context of the frame."""
-        if isinstance(code, basestring):
-            if isinstance(code, unicode):
+        if isinstance(code, string_types):
+            if PY2 and isinstance(code, unicode):
                 code = UTF8_COOKIE + code.encode('utf-8')
             code = compile(code, '<interactive>', mode)
-        if mode != 'exec':
-            return eval(code, self.globals, self.locals)
-        exec code in self.globals, self.locals
+        return eval(code, self.globals, self.locals)
 
     @cached_property
     def sourcelines(self):
@@ -431,7 +460,7 @@ class Frame(object):
 
         if source is None:
             try:
-                f = file(self.filename)
+                f = open(self.filename)
             except IOError:
                 return []
             try:
@@ -440,7 +469,7 @@ class Frame(object):
                 f.close()
 
         # already unicode?  return right away
-        if isinstance(source, unicode):
+        if isinstance(source, text_type):
             return source.splitlines()
 
         # yes. it should be ascii, but we don't want to reject too many
